#!/usr/bin/env ruby

##
# UPnPwn v0.8.0-bh
# Copyright 2008 Jonathan Squire, CISSP
##


require 'getoptlong'
require 'ssdp.rb'
require 'upnp.rb'
require 'socket'
require 'util'

UPNPWN_VERSION="0.8.0-bh"
#Disable reverse lookups
Socket.do_not_reverse_lookup = true


def help
	help_str = ""
	help_str << "Usage:\n"
	help_str << "  #{$0} [--url URL_to_IGD_desc]"
	help_str << "See README for additional usage"
	
	puts help_str
end


def upnpwn_list(igd)
	target_service='urn:schemas-upnp-org:service:WANIPConnection:1'	
	list_portmaps(igd.service[target_service])
end	


def upnpwn_add_help
	help_str = "ERROR bad arguments, expected one of the following:\n"
	
	help_str << "ExternalPort Protocol InternalClient\n"
	help_str << "   #{$0} --portmap add "
	help_str << "1234 TCP 192.168.88.11\n"
	
	help_str << "ExternalPort Protocol InternalPort InternalClient\n"
	help_str << "   #{$0} --portmap add "
	help_str << "1234 UDP 4321 192.168.88.11\n"

	help_str << "RemoteHost ExternalPort Protocol InternalPort InternalClient\n"
	help_str << "   #{$0} --portmap add "
	help_str << "1.22.33.44 1234 TCP 4321 192.168.88.11\n"

	help_str << "RemoteHost ExternalPort Protocol InternalPort InternalClient Enabled Description LeaseDuration\n"
	help_str << "   #{$0} --portmap add "
	help_str << '1.22.33.44 1234 TCP 4321 192.168.88.11 1 "Example 8" 0' + "\n"

	help_str << "   #{$0} --portmap add"
	help_str << '"" 1234 TCP 4321 192.168.88.11 0 "Example 8" 44' + "\n"

	puts help_str
end


def upnpwn_add(igd)
	target_service='urn:schemas-upnp-org:service:WANIPConnection:1'	
	
	rule_ok=false
	
	# NewExternalPort,NewProtocol,NewInternalClient
	if(ARGV.length==3)
		_NewRemoteHost=""
		_NewExternalPort=ARGV[0].to_i
		_NewProtocol=ARGV[1].to_s
		_NewInternalPort=_NewExternalPort
		_NewInternalClient=ARGV[2].to_s
		_NewEnabled=1
		_NewPortMappingDescription="UPnPwn: #{_NewExternalPort} to #{_NewInternalClient}:#{_NewInternalPort}"
		_NewLeaseDuration=0
		
		rule_ok=true
	end

	# NewExternalPort,NewProtocol,NewInternalPort,NewInternalClient
	if(ARGV.length==4)
		_NewRemoteHost=""
		_NewExternalPort=ARGV[0].to_i
		_NewProtocol=ARGV[1].to_s
		_NewInternalPort=ARGV[2].to_i
		_NewInternalClient=ARGV[3].to_s
		_NewEnabled=1
		_NewPortMappingDescription="UPnPwn: #{_NewExternalPort} to #{_NewInternalClient}:#{_NewInternalPort}"
		_NewLeaseDuration=0
		
		rule_ok=true
	end

	# NewRemoteHost,NewExternalPort,NewProtocol,NewInternalPort,NewInternalClient
	if(ARGV.length==5)
		_NewRemoteHost=ARGV[0].to_s
		_NewExternalPort=ARGV[1].to_i
		_NewProtocol=ARGV[2].to_s
		_NewInternalPort=ARGV[3].to_i
		_NewInternalClient=ARGV[4].to_s
		_NewEnabled=1
		_NewPortMappingDescription="UPnPwn: #{_NewExternalPort} to #{_NewInternalClient}:#{_NewInternalPort}"
		_NewLeaseDuration=0
		
		rule_ok=true
	end

	#NewRemoteHost,NewExternalPort,NewProtocol,NewInternalPort,NewInternalClient,NewEnabled,NewPortMappingDescription,NewLeaseDuration
	if(ARGV.length==8)
		_NewRemoteHost=ARGV[0].to_s
		_NewExternalPort=ARGV[1].to_i
		_NewProtocol=ARGV[2].to_s
		_NewInternalPort=ARGV[3].to_i
		_NewInternalClient=ARGV[4].to_s
		_NewEnabled=ARGV[5].to_i
		_NewPortMappingDescription=ARGV[6].to_s
		_NewLeaseDuration=ARGV[7].to_i
		
		rule_ok=true
	end

	if(rule_ok)
		puts "AddPortMapping('#{_NewRemoteHost}',#{_NewExternalPort},#{_NewProtocol},#{_NewInternalPort},#{_NewInternalClient},#{_NewEnabled},#{_NewPortMappingDescription},#{_NewLeaseDuration})"
		igd.service[target_service].AddPortMapping(_NewRemoteHost,_NewExternalPort,_NewProtocol,_NewInternalPort,_NewInternalClient,_NewEnabled,_NewPortMappingDescription,_NewLeaseDuration)
	else
		upnpwn_add_help
		exit
	end
end	


def upnpwn_del_help
	help_str = "ERROR bad arguments, expected one of the following:\n"

	help_str << "ExternalPort Protocol\n"
	help_str << "   #{$0} --portmap del "
	help_str << "1234 UDP\n"
	
	help_str << "RemoteHost ExternalPort Protocol\n"
	help_str << "   #{$0} --portmap del "
	help_str << "1.22.33.44 1234 TCP\n"
	
	puts help_str
end

#DeletePortMapping(NewRemoteHost,NewExternalPort,NewProtocol)
def upnpwn_del(igd, target_service='urn:schemas-upnp-org:service:WANIPConnection:1')

	del_ok=false
	
	# ExternalPort Protocol
	if(ARGV.length==2)
		_NewRemoteHost=""
		_NewExternalPort=ARGV[0].to_i
		_NewProtocol=ARGV[1].to_s
		del_ok=true
	end
	
	#RemoteHost ExternalPort Protocol
	if(ARGV.length==3)
		_NewRemoteHost=ARGV[0].to_s
		_NewExternalPort=ARGV[1].to_i
		_NewProtocol=ARGV[2].to_s
		del_ok=true
	end

	
	if(del_ok)
		igd.service[target_service].DeletePortMapping(_NewRemoteHost,_NewExternalPort,_NewProtocol)
	else
		upnpwn_del_help
		exit
	end
end

def upnpwn_ssdp_listen
	s = SSDP.new
	s.ssdp_listen
end

#
# Gets information about an IGD
#
def upnpwn_info(igd)
	puts igd.info
	igd.service.each {
		|key, value|
		value.load_xml_data
		value.parse_scpd
		value.soap_init
		puts value.info
		puts "\n"
	}

end

def upnpwn_test_server(host = Socket::INADDR_ANY, port = 1234)
	Socket.do_not_reverse_lookup = true
	
	
	server = host ? TCPServer.open(host, port) : TCPServer.open(port)
	
	port = server.addr[1]
	addrs = server.addr[2..-1].uniq
	
	puts "*** listening on #{addrs.collect{|a|"#{a}:#{port}"}.join(' ')}"
	
	socket = server.accept
	
    port = socket.peeraddr[1]
    name = socket.peeraddr[2]
    addr = socket.peeraddr[3]

    puts "*** recieving from #{name}:#{port}"
	socket.write("UPnPwn\nHello: #{name} (#{addr}):#{port}\n")
	socket.close
end
	
#
#  Main Block
#
def upnpwn_main
	puts "UPnPwn v#{UPNPWN_VERSION}"
	
	opts = GetoptLong.new(
		[ "--info", GetoptLong::NO_ARGUMENT],				# Get info about an IGD
		[ "--list", GetoptLong::NO_ARGUMENT],				# LIST Portmaps
		[ "--portmap", GetoptLong::REQUIRED_ARGUMENT],		# ADD/DEL/LIST Portmap
		[ "--ssdp_listen", GetoptLong::NO_ARGUMENT],		#Listen for SSDP devices
		[ "--test_server", GetoptLong::NO_ARGUMENT],		#UPnPwn Test Server
		[ "--url", "-u", GetoptLong::REQUIRED_ARGUMENT],	# URL to device desc
		[ "--help", "-h", GetoptLong::NO_ARGUMENT]			#display help
	)

	url=nil
	portmap_mode=nil
	portmap=false
	info=false
	datacap=false
	need_igd=false
	test_server=false

	#
	# process arguments
	#
	opts.each {
		|opt,arg|
		case opt
			when '--list'
				need_igd=true
				portmap=true
				portmap_mode = "list"
			when '--portmap'
				need_igd=true
				portmap=true
				portmap_mode = arg
			when '--ssdp_listen'
				upnpwn_ssdp_listen
			when '--test_server'
				test_server=true
			when '--info'
				need_igd=true
				info = true
			when '--datacap'
				need_igd=true
				datacap = true
			when '--url'
				url = arg
			when '--help'
				need_igd=false
				help
		end
	}
	
	#
	# Get an IGD if we need it
	#
	if(need_igd)
		if(url.nil?)
			puts "Finding an IGD..."
			s = SSDP.new
			url = s.find_IGD
		end
		
		puts "IGD Description: #{url}"

		igd=UPnPHost.new
		igd.location=url
		igd.load_xml_data
		igd.parse_services

		igd.service.each {
			|key, value|
			value.load_xml_data
			value.parse_scpd
			value.soap_init
		}
	end

	if(portmap)
		case portmap_mode
			when 'list'
				upnpwn_list(igd)
			when 'add'
				upnpwn_add(igd)
			when 'del'
				upnpwn_del(igd)
			else
				puts "Unknown Portmap Command: #{portmap_mode}"
				help
				exit
		end
	end
	
	if(info)
		upnpwn_info(igd)
	end
	
	if(test_server)
		upnpwn_test_server
	end
end


upnpwn_main



