#!/usr/bin/env ruby

##
# UPnPwn v0.8.0-bh
# Copyright 2008 Jonathan Squire, CISSP
##


require 'getoptlong'
require 'upnp.rb'

def header(classname="UNDEFINED")

	h = ""
	h << "#!/usr/bin/env ruby\n"
	h << "\n"

	h << "=begin\n"
	h << "scpd2ruby Automatically generated\n\n"
	h << $s.info
	h << "\n\n"
	h << "=end\n\n\n"
	
	h << "require 'soap/rpc/driver'\n"
	h << "\n"
	h << "class #{classname}\n"
	h << "\n"
	h << "attr_accessor :soap\n"
	h << "\n"
	
	## require a control_url
	h << "def initialize(control_url,namespace"
	## if namespace is know, make it the default
	if (!$s.serviceType.nil?)
		h << "='#{$s.serviceType}'"
	end
	h << ")\n"
	h << "meth_url=control_url\n"
	h << "meth_namespace=namespace\n"
	h << "self.soap = SOAP::RPC::Driver.new(meth_url,meth_namespace)\n"
	h << "\n\n"

	return h
end

def footer
	f = ""
	f << "end\n"
	f << "end\n"

	return f
end

#
#  Main Block
#

opts = GetoptLong.new(
	[ "--scpdfile", "-f", GetoptLong::REQUIRED_ARGUMENT],
	[ "--url", "-u", GetoptLong::REQUIRED_ARGUMENT],
	[ "--outfile", "-o", GetoptLong::REQUIRED_ARGUMENT],
	[ "--classname", "-c", GetoptLong::REQUIRED_ARGUMENT],
	[ "--namespace", "-n", GetoptLong::REQUIRED_ARGUMENT],
	[ "--nocomments", GetoptLong::NO_ARGUMENT],
	[ "--help", "-h", GetoptLong::NO_ARGUMENT]
)

scpdfile=nil
outfile=nil
classname="SCPDService"
comments=true
url=nil
scpd_XML=nil
namespace=nil

opts.each {
	|opt,arg|
	case opt
		when '--help'
			puts "Help"
		when '--scpdfile'
			scpdfile = arg
		when '--outfile'
			outfile = arg
		when '--classname'
			classname = arg
		when '--nocomments'
			comments = false
		when '--namespace'
			namespace = arg
		when '--url'
			url = arg
	end
}

  
  $s=UPnPService.new
  
  if(!url.nil?)
	$s.xml_data=Net::HTTP.get_response(URI.parse(url)).body
  else
	url = "file://"
	if(!scpdfile.nil?)
		f=File.open(scpdfile, 'r')
		url << scpdfile
	else
		f=STDIN
		url << "STDIN"
	end
	$s.xml_data=f.read
	f.close	
  end
  
  $s.scpdURL=url
  $s.serviceType=namespace

  $s.parse_scpd
      
  puts header(classname)
  $s.print_actions
  puts footer
